@@ -68,7 +68,9 @@ sub methods {
 
         my $sym_table = $class->mc->symbol_table;
         for my $name (keys %$sym_table) {
-            next unless *{$sym_table->{$name}}{CODE};
+            my $glob = $sym_table->{$name};
+            next unless ref \$glob eq "GLOB";
+            next unless *{$glob}{CODE};
             $all_methods{$name} = $class;
         }
     }
