@@ -25,6 +25,8 @@ sub alias {
 sub is_number   { return }
 sub is_positive { return }
 sub is_negative { return }
+sub is_even     { return }
+sub is_odd      { return }
 sub is_int      { return }
 sub is_integer  { return }
 sub is_decimal  { return }
